/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: amvsReg.c,v 5.1 1999/05/04 12:37:27 matthias Exp $
 *
 */

/*
 * Defines:
 */

/* #define DEBUG_ICPCI_REG */

/*
 * Includes:
 */

#include "icpciOS.h"	/* os specific stuff */
#include "icpciExt.h"	/* offical driver interface */
#include "icpciInt.h"	/* driver internals */
#include "amvsReg.h"	/* register definitions AM-VS */

/*
 * Globals:
 */

extern struct
icp_global_t    icp;	/* main driver structure */

#if (DEBUG_ICPCI || DEBUG_ICPCI_REG)
const char  *vs_reg[] = { "modid ",
			  "con1  ",
			  "con2  ",
			  "acqcon",
			  "hoff  ",
			  "hsz   ",
			  "voff  ",
			  "vsz   ",
			  "froff ",
			  "cpcon ",
			  "swtrig",
			  "error ",
			  "error ",
			  "error ",
			  "error ",
			  "error ",
			  "lutadr",
			  "lutdat",
			  "adccon"
			};
#endif /* DEBUG_ICPCI_REG */

/*
 * Prototypes:
 */

/*
 * General Read Handler for AM-VS Registers
 */

/* 16 bit register access with mask and range checking */

static u_short
vs_read_mm (board, offset, mask, min, max)
     
  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_char  offset;	/* offset to register base address pointer */
  u_short mask;		/* read mask for register */
  u_short min;		/* minimal value for range checking */
  u_short max;		/* maximal value for range checking */

{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset / 
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  if (offset != VS_MODID_OFFSET)
    tmp_val &= VS_MAGIC_MASK;

  tmp_val &= mask;

#if(0)
  if (tmp_val & ~mask) icp.errno[board] = -EICP_MASK;

  if (tmp_val < min || tmp_val > max) icp.errno[board] = -EICP_RANGE;
#endif

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  vs-readw  %s, %#06X &(%#06X) >%#06X <%#06X\n",
	     board, vs_reg[offset], tmp_val, mask, min, max);
#endif

  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_char
vs_read_ms (board, offset, mask, shift)

  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_char  offset;	/* offset to register base address pointer */
  u_char  shift;	/* number of bits to shift right */
  u_short mask;		/* read mask for register */
  
{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  if (offset != VS_MODID_OFFSET)
    tmp_val &= VS_MAGIC_MASK;

  tmp_val &= mask;
  tmp_val >>= shift;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  vs-readw  %s, %#1X &(%#06X) >>%2d\n",
	     board, vs_reg[offset], tmp_val, mask, shift);
#endif

  return ((u_char)tmp_val);
}
  
/*
 * General Write Handler for AM-VS Registers
 */

/* 16 bit register access with mask and range checking */

static int
vs_write_mm (board, offset, mask, min, max, value)

  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_char  offset;	/* offset to register base address pointer */
  u_short mask;		/* read mask for register */
  u_short min;		/* minimal value for range checking */
  u_short max;		/* maximal value for range checking */
  u_short value;	/* value to set to */

{
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  if (value & ~mask) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  if (value < min || value > max) {
    icp.errno[board] = -EICP_RANGE;  
    return (ERROR);
  }

  tmp_val = value & mask;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  vs-writew %s, %#06X &(%#06X) >%#06X <%#06X\n",
	     board, vs_reg[offset], tmp_val, mask, min, max);
#endif

  tmp_val |= ~VS_MAGIC_MASK;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  icp_writew (tmp_val, icp.map[board]->v_mod_base + offset);

  return (OK);
}

/* 16 bit register 1 (or more) bit access */

static int
vs_write_ms (board, offset, mask, shift, value)

  u_char  board;	/* number of IC-PCI board attached to the computer */
  u_char  offset;	/* offset to register base address pointer */
  u_short mask;		/* read mask for register */
  u_char  shift;	/* number of bits to shift right */
  u_char  value;	/* value to set to */  
  
{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;
  u_short amcc_dummy = 0;

  tmp_val = (u_short)value << shift;

  if (tmp_val & ~mask) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  read_val = icp_readw (icp.map[board]->v_mod_base + offset);

  if (offset != VS_MODID_OFFSET)
    read_val &= VS_MAGIC_MASK;

  read_val &= ~mask;
  write_val = read_val | tmp_val;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  vs-writew %s, %#06X->%#06X %#1X &(%#06X) <<%2d\n",
	     board, vs_reg[offset], read_val, write_val, tmp_val, mask, shift);
#endif

  write_val |= ~VS_MAGIC_MASK;

  if (icp.cap[board]->amcc0_bug) {
    if (icp_check_amcc (board, icp.map[board]->v_cr_base + offset /
			sizeof(u_short), ICP_AMCC_READ)) {
      ICP_DELAY(icp.cap[board]->mem_wait16);
      amcc_dummy = icp_readw (icp.map[board]->v_cr_base + offset /
			  sizeof(u_short));
    }
  }
  if (icp.cap[board]->amcc1_bug) {
    ICP_DELAY(icp.cap[board]->mem_wait16);
  }

  icp_writew (write_val, icp.map[board]->v_mod_base + offset);

  return (OK);
}
  
/*
 * Special Read Handler for AM-VS Registers
 */

u_char
vs_get_rev (u_char board) {

  return (vs_read_ms (board,
		      VS_MODID_OFFSET,
		      VS_REV_MASK,
		      VS_REV_SHIFT));
}

u_char
vs_get_tgid (u_char board) {

  u_char tmp_val = 0;
  
  tmp_val = vs_read_ms (board,
			VS_MODID_OFFSET,
			VS_TGID_MASK,
			VS_TGID_SHIFT);

#if(0)
  if (tmp_val > 2) icp.errno[board] = -EICP_RANGE;
#endif

  return (tmp_val);
}

u_char
vs_get_tsel (u_char board) {

  u_char tgid = 0, tsel = 0;
  
  tgid = vs_get_tgid (board);
  
  tsel = vs_read_ms (board,
		     VS_CON1_OFFSET,
		     VS_TSEL_MASK,
		     VS_TSEL_SHIFT);

#if(0)
  switch (tgid) {
  case 1:
    if (tsel == 5) icp.errno[board] = -EICP_RANGE;
    break;
  case 0:
    if (tsel == 1 || tsel == 5) icp.errno[board] = -EICP_RANGE;	
    break;
  }
#endif
  
  return (tsel);
}

u_char
vs_get_clksrc (u_char board) {

  return (vs_read_ms (board,
		      VS_CON1_OFFSET,
		      VS_CLKSRC_MASK,
		      VS_CLKSRC_SHIFT));
}

u_char
vs_get_lpfsel (u_char board) {

  return (vs_read_ms (board,
		      VS_CON1_OFFSET,
		      VS_LPFSEL_MASK,
		      VS_LPFSEL_SHIFT));
}

u_char
vs_get_iscale (u_char board) {

  return (vs_read_ms (board,
		      VS_CON1_OFFSET,
		      VS_ISCALE_MASK,
		      VS_ISCALE_SHIFT));
}

u_char
vs_get_smode (u_char board) {

  return (vs_read_ms (board,
		      VS_CON1_OFFSET,
		      VS_SMODE_MASK,
		      VS_SMODE_SHIFT));
}

#if(0)	/* ? not supported ? */
u_char
vs_get_camsel (u_char board) {

  return (vs_read_ms (board,
		      VS_CON1_OFFSET,
		      VS_CAMSEL_MASK,
		      VS_CAMSEL_SHIFT));
}
#endif

u_char
vs_get_tsrc (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_TSRC_MASK,
		      VS_TSRC_SHIFT));
}

u_char
vs_get_lenpol (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_LENPOL_MASK,
		      VS_LENPOL_SHIFT));
}

u_char
vs_get_fldpol (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_FLDPOL_MASK,
		      VS_FLDPOL_SHIFT));
}

u_char
vs_get_fenpol (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_FENPOL_MASK,
		      VS_FENPOL_SHIFT));
}

u_char
vs_get_pclkpol (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_PCLKPOL_MASK,
		      VS_PCLKPOL_SHIFT));
}

u_char
vs_get_eclken (u_char board) {

  u_char tsel = 0, eclken = 0;
  
  tsel = vs_get_tsel (board);
  
  eclken = vs_read_ms (board,
		       VS_CON2_OFFSET,
		       VS_ECLKEN_MASK,
		       VS_ECLKEN_SHIFT);
  
#if(0)
  switch (tsel) {
  case 5:
    icp.errno[board] = -EICP_RANGE;
    break;
  case 1:
    if (eclken) icp.errno[board] = -EICP_RANGE;
    break;
  }
#endif

  return (eclken);
}

u_char
vs_get_vbstat (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_VBSTAT_MASK,
		      VS_VBSTAT_SHIFT));
}

u_char
vs_get_fldsten (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_FLDSTEN_MASK,
		      VS_FLDSTEN_SHIFT));
}

u_char
vs_get_vsten (u_char board) {

  return (vs_read_ms (board,
		      VS_CON2_OFFSET,
		      VS_VSTEN_MASK,
		      VS_VSTEN_SHIFT));
}

u_char
vs_get_trigen (u_char board) {

  return (vs_read_ms (board,
		      VS_TRIGCON_OFFSET,
		      VS_TRIGEN_MASK,
		      VS_TRIGEN_SHIFT));
}

u_char
vs_get_trigsrc (u_char board) {

  return (vs_read_ms (board,
		      VS_TRIGCON_OFFSET,
		      VS_TRIGSRC_MASK,
		      VS_TRIGSRC_SHIFT));
}

u_char
vs_get_fmode  (u_char board) {

#if(0)
  if (vs_get_rev (board) < 1) icp.errno[board] = -EICP_RANGE;
#endif

  return (vs_read_ms (board,
		      VS_TRIGCON_OFFSET,
		      VS_FMODE_MASK,
		      VS_FMODE_SHIFT));
}

u_char
vs_get_tmode (u_char board) {

  return (vs_read_ms (board,
		      VS_TRIGCON_OFFSET,
		      VS_TMODE_MASK,
		      VS_TMODE_SHIFT));
}

u_char
vs_get_trigcy (u_char board) {

  return (vs_read_ms (board,
		      VS_TRIGCON_OFFSET,
		      VS_TRIGCY_MASK,
		      VS_TRIGCY_SHIFT));
}

u_char
vs_get_strbdly (u_char board) {

  return (vs_read_ms (board,
		      VS_TRIGCON_OFFSET,
		      VS_STRBDLY_MASK,
		      VS_STRBDLY_SHIFT));
}

u_short
vs_get_hoff (u_char board) {

  return (vs_read_mm (board,
		      VS_HOFF_OFFSET,
		      VS_HOFF_MASK,
		      VS_HOFF_MIN,
		      VS_HOFF_MAX));
}

u_short
vs_get_hsz (u_char board) {

  u_short tmp_val = 0; 

  tmp_val = vs_read_mm (board,
			VS_HSZ_OFFSET,
			VS_HSZ_MASK,
			VS_HSZ_MIN,
			VS_HSZ_MAX);
  
  return ((tmp_val + 1) << 2);
}

u_short
vs_get_voff (u_char board) {

  return (vs_read_mm (board,
		      VS_VOFF_OFFSET,
		      VS_VOFF_MASK,
		      VS_VOFF_MIN,
		      VS_VOFF_MAX));
}

u_short
vs_get_vsz (u_char board) {

  u_short tmp_val = 0; 

  tmp_val = vs_read_mm (board,
			VS_VSZ_OFFSET,
			VS_VSZ_MASK,
			VS_VSZ_MIN,
			VS_VSZ_MAX);

  return (tmp_val + 1);
}

u_short
vs_get_froff (u_char board) {

  return (vs_read_mm (board,
		      VS_FROFF_OFFSET,
		      VS_FROFF_MASK,
		      VS_FROFF_MIN,
		      VS_FROFF_MAX));
}

u_char
vs_get_cppos (u_char board) {

  return (vs_read_ms (board,
		      VS_CPCON_OFFSET,
		      VS_CPPOS_MASK,
		      VS_CPPOS_SHIFT));
}

u_char
vs_get_cpen (u_char board) {

  return (vs_read_ms (board,
		      VS_CPCON_OFFSET,
		      VS_CPEN_MASK,
		      VS_CPEN_SHIFT));
}

u_char
vs_get_cpsrc (u_char board) {

  return (vs_read_ms (board,
		      VS_CPCON_OFFSET,
		      VS_CPSRC_MASK,
		      VS_CPSRC_SHIFT));
}

u_char
vs_get_strbsel (u_char board) {

  return (vs_read_ms (board,
		      VS_CPCON_OFFSET,
		      VS_STRBSEL_MASK,
		      VS_STRBSEL_SHIFT));
}

u_char
vs_get_strbpol (u_char board) {

  return (vs_read_ms (board,
		      VS_CPCON_OFFSET,
		      VS_STRBPOL_MASK,
		      VS_STRBPOL_SHIFT));
}
			
u_char
vs_get_cpst (u_char board) {

#if(0)
  if (vs_get_rev (board) < 2) icp.errno[board] = -EICP_RANGE;
#endif

  return (vs_read_ms (board,
		      VS_CPCON_OFFSET,
		      VS_CPST_MASK,
		      VS_CPST_SHIFT));
}

u_char
vs_get_lutdata (u_char board, struct vs_lut_t *get_lut) {

  int count;

  vs_write_mm (board,
	       VS_LUTADDR_OFFSET,
	       VS_LUTADDR_MASK,
	       VS_LUTADDR_MIN,
	       VS_LUTADDR_MAX,
	       VS_LUTADDR_MIN);

#if(0)
  if (get_lut == NULL) icp.errno[board] = -EICP_POINTER;
  if (icp.errno[board] < 0) return (ERROR);
#endif

  for (count = VS_LUTADDR_MIN; count <= VS_LUTADDR_MAX; count++) {

    get_lut->lutdata[count] = vs_read_mm (board,
					  VS_LUTDATA_OFFSET,
					  VS_LUTDATA_MASK,
					  VS_LUTDATA_MIN,
					  VS_LUTDATA_MAX);

#if(0)    
    if (icp.errno[board] < 0) return (ERROR);
#endif

  }
  return (OK);
}

u_short
vs_get_nref (u_char board) {

  vs_write_mm (board,
	       VS_LUTADDR_OFFSET,
	       VS_LUTADDR_MASK,
	       VS_LUTADDR_MIN,
	       VS_LUTADDR_MAX,
	       VS_LUTADDR_ADC_NR);

#if(0)  
  if (icp.errno[board] < 0) return (0);
#endif
  
  return (vs_read_mm (board,
		      VS_NREF_OFFSET,
		      VS_NREF_MASK,
		      VS_NREF_MIN,
		      VS_NREF_MAX));
}

u_short
vs_get_pref (u_char board) {

  vs_write_mm (board,
	       VS_LUTADDR_OFFSET,
	       VS_LUTADDR_MASK,
	       VS_LUTADDR_MIN,
	       VS_LUTADDR_MAX,
	       VS_LUTADDR_ADC_PR);

#if(0)  
  if (icp.errno[board] < 0) return (0);
#endif
  
  return (vs_read_mm (board,
		      VS_PREF_OFFSET,
		      VS_PREF_MASK,
		      VS_PREF_MIN,
		      VS_PREF_MAX));
}
			
/*
 * Special Write Handler for AM-VS Registers
 */

int
vs_set_tsel (u_char board, u_char value) {

    u_char tgid = 0;

    tgid = vs_get_tgid (board);

    switch (tgid) {
    case 1:
	if (value == 5) {
	    icp.errno[board] = -EICP_NOSUPPORT;
	    return (ERROR);
	}
	break;
    case 0:
	if (value == 1 || value == 5) {
	    icp.errno[board] = -EICP_NOSUPPORT;
	    return (ERROR);
	}
	break;
    }	

    return (vs_write_ms (board,
			 VS_CON1_OFFSET,
			 VS_TSEL_MASK,
			 VS_TSEL_SHIFT,
			 value));
}

int
vs_set_clksrc (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON1_OFFSET,
		       VS_CLKSRC_MASK,
		       VS_CLKSRC_SHIFT,
		       value));
}

int
vs_set_lpfsel (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON1_OFFSET,
		       VS_LPFSEL_MASK,
		       VS_LPFSEL_SHIFT,
		       value));
}

int
vs_set_iscale (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON1_OFFSET,
		       VS_ISCALE_MASK,
		       VS_ISCALE_SHIFT,
		       value));
}

int
vs_set_smode (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON1_OFFSET,
		       VS_SMODE_MASK,
		       VS_SMODE_SHIFT,
		       value));
}

int
vs_set_camsel (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON1_OFFSET,
		       VS_CAMSEL_MASK,
		       VS_CAMSEL_SHIFT,
		       value));
}

int
vs_set_tsrc (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_TSRC_MASK,
		       VS_TSRC_SHIFT,
		       value));
}

int
vs_set_lenpol (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_LENPOL_MASK,
		       VS_LENPOL_SHIFT,
		       value));
}

int
vs_set_fldpol (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_FLDPOL_MASK,
		       VS_FLDPOL_SHIFT,
		       value));
}

int
vs_set_fenpol (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_FENPOL_MASK,
		       VS_FENPOL_SHIFT,
		       value));
}

int
vs_set_pclkpol (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_PCLKPOL_MASK,
		       VS_PCLKPOL_SHIFT,
		       value));
}

int
vs_set_eclken (u_char board, u_char value) {

  u_char tsel = 0;
  
  tsel = vs_get_tsel (board);
  
  switch (tsel) {
  case 5:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  case 1:
    if (value) {
      icp.errno[board] = -EICP_NOSUPPORT;
      return (ERROR);
    }
    break;
  }
  
  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_ECLKEN_MASK,
		       VS_ECLKEN_SHIFT,
		       value));
  
  return (OK);
}

int
vs_set_fldsten (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_FLDSTEN_MASK,
		       VS_FLDSTEN_SHIFT,
		       value));
}

int
vs_set_vsten (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CON2_OFFSET,
		       VS_VSTEN_MASK,
		       VS_VSTEN_SHIFT,
		       value));
}

int
vs_set_trigen (u_char board, u_char value) {

  if (value == VS_ENABLE) {
    if (vs_get_trigcy (board) == VS_IN_TRIGCY) {
      icp.errno[board] = -EICP_BUSY;
      return (ERROR);
    }
  }

  return (vs_write_ms (board,
		       VS_TRIGCON_OFFSET,
		       VS_TRIGEN_MASK,
		       VS_TRIGEN_SHIFT,
		       value));
}

int
vs_set_trigsrc (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_TRIGCON_OFFSET,
		       VS_TRIGSRC_MASK,
		       VS_TRIGSRC_SHIFT,
		       value));
}

int
vs_set_fmode (u_char board, u_char value) {

    if (vs_get_rev (board) < 1) {
	icp.errno[board] = -EICP_NOSUPPORT;
	return (ERROR);
    }

    return (vs_write_ms (board,
			 VS_TRIGCON_OFFSET,
			 VS_FMODE_MASK,
			 VS_FMODE_SHIFT,
			 value));
}

int
vs_set_tmode (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_TRIGCON_OFFSET,
		       VS_TMODE_MASK,
		       VS_TMODE_SHIFT,
		       value));
}

int
vs_set_strbdly (u_char board, u_char value) {

  if (value > VS_STRBDLY_MAX) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }
  
  return (vs_write_ms (board,
		       VS_TRIGCON_OFFSET,
		       VS_STRBDLY_MASK,
		       VS_STRBDLY_SHIFT,
		       value));
}

int
vs_set_hoff (u_char board, u_short value) {

  return (vs_write_mm (board,
		       VS_HOFF_OFFSET,
		       VS_HOFF_MASK,
		       VS_HOFF_MIN,
		       VS_HOFF_MAX,
		       value));
}

int
vs_set_hsz (u_char board, u_short value) {

  return (vs_write_mm (board,
		       VS_HSZ_OFFSET,
		       VS_HSZ_MASK,
		       VS_HSZ_MIN,
		       VS_HSZ_MAX,
		       (value - 1) >> 2));
}

int
vs_set_voff (u_char board, u_short value) {

  return (vs_write_mm (board,
		       VS_VOFF_OFFSET,
		       VS_VOFF_MASK,
		       VS_VOFF_MIN,
		       VS_VOFF_MAX,
		       value));
}

int
vs_set_vsz (u_char board, u_short value) {

  return (vs_write_mm (board,
		       VS_VSZ_OFFSET,
		       VS_VSZ_MASK,
		       VS_VSZ_MIN,
		       VS_VSZ_MAX,
		       value - 1));
}

int
vs_set_froff (u_char board, u_short value) {

  return (vs_write_mm (board,
		       VS_FROFF_OFFSET,
		       VS_FROFF_MASK,
		       VS_FROFF_MIN,
		       VS_FROFF_MAX,
		       value));
}

int
vs_set_cppos (u_char board, u_char value) {

  if (value > VS_CPPOS_MAX) {
    icp.errno[board] = -EICP_RANGE;
    return (ERROR);
  }

  return (vs_write_ms (board,
		       VS_CPCON_OFFSET,
		       VS_CPPOS_MASK,
		       VS_CPPOS_SHIFT,
		       value));
}

int
vs_set_cpen (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CPCON_OFFSET,
		       VS_CPEN_MASK,
		       VS_CPEN_SHIFT,
		       value));
}

int
vs_set_cpsrc (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CPCON_OFFSET,
		       VS_CPSRC_MASK,
		       VS_CPSRC_SHIFT,
		       value));
}

int
vs_set_strbsel (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CPCON_OFFSET,
		       VS_STRBSEL_MASK,
		       VS_STRBSEL_SHIFT,
		       value));
}

int
vs_set_strbpol (u_char board, u_char value) {

  return (vs_write_ms (board,
		       VS_CPCON_OFFSET,
		       VS_STRBPOL_MASK,
		       VS_STRBPOL_SHIFT,
		       value));
}

int
vs_set_cpst (u_char board, u_char value) {

  if (vs_get_rev (board) < 2) {
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
  }


  return (vs_write_ms (board,
		       VS_CPCON_OFFSET,
		       VS_CPST_MASK,
		       VS_CPST_SHIFT,
		       value));
}

int
vs_set_lutdata (u_char board, struct vs_lut_t *set_lut) {

  int count;

  if (set_lut == NULL) {
    icp.errno[board] = -EICP_POINTER;
    return (ERROR);
  }

  if (vs_write_mm (board,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_MIN) < 0) return (ERROR);
   
  for (count = VS_LUTADDR_MIN; count <= VS_LUTADDR_MAX; count++) {

  if (vs_write_mm (board,
		   VS_LUTDATA_OFFSET,
		   VS_LUTDATA_MASK,
		   VS_LUTDATA_MIN,
		   VS_LUTDATA_MAX,
		   set_lut->lutdata[count]) < 0) return (ERROR);
  }
  return (OK);
}

int
vs_set_nref (u_char board, u_short value) {

  if (vs_write_mm (board,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_ADC_NR) < 0) return (ERROR);

  return (vs_write_mm (board,
		       VS_NREF_OFFSET,
		       VS_NREF_MASK,
		       VS_NREF_MIN,
		       VS_NREF_MAX,
		       value));
}

int
vs_set_pref (u_char board, u_short value) {

  if (vs_write_mm (board,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_ADC_PR) < 0) return (ERROR);
  
  return (vs_write_mm (board,
		       VS_PREF_OFFSET,
		       VS_PREF_MASK,
		       VS_PREF_MIN,
		       VS_PREF_MAX,
		       value));
}

/*
 * Special Initialization Handler for AM-VS Registers
 */

int
vs_init_con1 (u_char board) {

  return (vs_write_mm (board,
		       VS_CON1_OFFSET,
		       VS_NO_MASK,
		       VS_NO_MIN,
		       VS_NO_MAX,
		       VS_RES_INIT));
}

int
vs_init_con2 (u_char board) {
    
  return (vs_write_mm (board,
		       VS_CON2_OFFSET,
		       VS_NO_MASK,
		       VS_NO_MIN,
		       VS_NO_MAX,
		       VS_RES_INIT));
}

int
vs_init_trigcon (u_char board) {
    
    return (vs_write_mm (board,
			 VS_TRIGCON_OFFSET,
			 VS_NO_MASK,
			 VS_NO_MIN,
			 VS_NO_MAX,
			 VS_RES_INIT));
}

int
vs_init_cpcon (u_char board) {
    
    return (vs_write_mm (board,
			 VS_CPCON_OFFSET,
			 VS_NO_MASK,
			 VS_NO_MIN,
			 VS_NO_MAX,
			 VS_RES_INIT));
}

int
vs_init_adccon (u_char board) {
    
  if (vs_write_mm (board,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_ADC_CR) < 0) return (ERROR);

  return (vs_write_mm (board,
		       VS_ADCCON_OFFSET,
		       VS_NO_MASK,
		       VS_NO_MIN,
		       VS_NO_MAX,
		       VS_ADCCON_INIT));
}

/* 
 * Miscellanous functions
 */
 
/* trigger per software */

int
vs_sw_trig (u_char board) {

  return (vs_write_mm (board,
		       VS_SWTRIG_OFFSET,
		       VS_SWTRIG_MASK,
		       VS_SWTRIG_MIN,
		       VS_SWTRIG_MAX,
		       VS_SWTRIG));
}

/* linearize lookup table */

int
vs_lin_lut (u_char board) {

  int count;

  if (vs_write_mm (board,
		   VS_LUTADDR_OFFSET,
		   VS_LUTADDR_MASK,
		   VS_LUTADDR_MIN,
		   VS_LUTADDR_MAX,
		   VS_LUTADDR_MIN) < 0) return (ERROR);

  for (count = VS_LUTADDR_MIN; count <= VS_LUTADDR_MAX; count++) {

    if (vs_write_mm (board,
		     VS_LUTDATA_OFFSET,
		     VS_LUTDATA_MASK,
		     VS_LUTDATA_MIN,
		     VS_LUTDATA_MAX,
		     count) < 0) return (ERROR);
  }
  return (OK);
}

#ifdef DEBUG_ICPCI
/* make register dump */
void
vs_dump_regs (u_char board) {
    
  int count;

  for (count = 0; count < VS_REG_SIZE; count++) {
    icp_printf ("icpci[%d]: %s %#04x\n", board, vs_reg[count],
		vs_read_mm (board, count, VS_NO_MASK, VS_NO_MIN, VS_NO_MAX));
  }
}
#endif

